#!/usr/bin/env Rscript
library(dplyr)
library(magrittr)

## Bioclim abbreviations
abbrev = c('bio_1'='AMT',
           'bio_2'='MDR',
           'bio_3'='IT',
           'bio_4'='TS',
           'bio_5'='TWaM',
           'bio_6'='TCM',
           'bio_7'='TAR',
           'bio_8'='TWeQ',
           'bio_9'='TDQ',
           'bio_10'='TWaQ',
           'bio_11'='TCQ',
           'bio_12'='AP',
           'bio_13'='PWeM',
           'bio_14'='PDM',
           'bio_15'='PS',
           'bio_16'='PWeQ',
           'bio_17'='PDQ',
           'bio_18'='PWaQ',
           'bio_19'='PCQ'
           )


## Read in the file with the bioclim data and take only the
## the individuals that are in our current genotyping set.
## Also trim down to just the bioclim columns
dat = read.csv('/home/tiffinp/epste051/project/metab_gwas/data/strain/2016-07-27/data_bioclim.tsv',
               sep='\t', header=TRUE, as.is=TRUE) %>%
filter(GWAS_meliloti == 'Y') %>%
filter(!is.na(bio_1)) %>%
select(starts_with('bio_')) %>%
as.matrix()

## Run a PCA using the standard R functions
## As recommended in the manual, I use prcomp instead of princomp,
## and I center and scale the matrix.
pca = prcomp(dat, center=TRUE, scale=TRUE)

pdf('bioclim_pca_loading_figure_2018-05-21.pdf', width=6.5, height=3.5)

par(mfrow=c(1, 2), mar=c(2, 2, 1, 1), mgp=c(0.5, 0, 0))

## Plot the loadings
bplot = function(pca, ax1, ax2) {

    xrange = max(pca$x[, ax1]) - min(pca$x[, ax1])
    yrange = max(pca$x[, ax2]) - min(pca$x[, ax2])
    rotxrange = max(pca$rotation[, ax1]) - min(pca$rotation[, ax1])
    rotyrange = max(pca$rotation[, ax2]) - min(pca$rotation[, ax2])
    ratio = max(xrange / rotxrange, yrange/rotyrange)
    xmin = min(min(pca$x[, ax1]), min(pca$rotation[, ax1] * ratio))
    xmax = max(max(pca$x[, ax1]), max(pca$rotation[, ax1] * ratio))
    ymin = min(min(pca$x[, ax2]), min(pca$rotation[, ax2] * ratio))
    ymax = max(max(pca$x[, ax2]), max(pca$rotation[, ax2] * ratio))
    pve = pca$sdev^2 / sum(pca$sdev^2)
    var_names = dimnames(pca$rotation)[[1]]

    x_rot = pca$rotation[, ax1] * ratio
    y_rot = pca$rotation[, ax2] * ratio
    d_rot = sqrt(x_rot^2 + y_rot^2)
    theta = atan2(y_rot, x_rot)

    plot(pca$x[, ax1], pca$x[, ax2], pch=19, cex=0.8, col='grey60',
         xlab=paste0('PC', ax1, ' (', round(pve[ax1] * 100, 0), '%)'),
         ylab=paste0('PC', ax2, ' (', round(pve[ax2] * 100, 0), '%)'),
         xlim=c(xmin * 1.1, xmax * 1.1), ylim=c(ymin * 1.1, ymax * 1.1),
         xaxt='n', yaxt='n')

    segments(0, 0, x_rot, y_rot,
             col=ifelse(as.numeric(gsub('bio_', '', var_names)) > 11,
                        'skyblue', 'indianred'),
             lwd=ifelse(var_names %in% c('bio_1', 'bio_12'), 1.5, 0.5))

    text((d_rot + 0.4) * cos(theta), (d_rot + 0.4) * sin(theta),
         labels=abbrev[dimnames(pca$rotation)[[1]]],
         col=ifelse(as.numeric(gsub('bio_', '', var_names)) > 11,
                    'skyblue', 'indianred'),
         font=ifelse(var_names %in% c('bio_1', 'bio_12'), 2, 1),
         cex=ifelse(var_names %in% c('bio_1', 'bio_12'), 0.75, 0.6),
         xpd=NA)

}

bplot(pca, 1, 2)
bplot(pca, 3, 4)

dev.off()
